<?php

namespace Garradin\Plugin\RecusFiscaux;

// génération du formulaire

class RecusHTML
{
    private $nomAsso;
    private $adresseAsso;
    private $logoAsso;
    private $objetAsso;
    private $nomResponsable;
    private $fonctionResponsable;
    private $villeAsso;
    private $articlesCGI;
    private $signature;

    /**
     * initialize global data
     */
    function __construct($nomAsso,
                         $adresseAsso,
                         $logoAsso,
                         $objetAsso,
                         $nomResponsable,
                         $fonctionResponsable,
                         $villeAsso,
                         $articlesCGI,
                         $signature)
    {
        $this->nomAsso = $nomAsso;
        $this->adresseAsso = $adresseAsso;
        $this->logoAsso = $logoAsso;
        $this->objetAsso = $objetAsso;
        $this->nomResponsable = $nomResponsable;
        $this->fonctionResponsable = $fonctionResponsable;
        $this->villeAsso = $villeAsso;
        $this->signature = $signature;
        $this->articlesCGI = $articlesCGI;
        $this->html = $this->entete();
    }

    function get()
    {
        return $this->html;
    }

    // imprimer le reçu
    function imprimer_recu($annee_recu,
                           $numero,
                           $nom,
                           $lesMontants,
                           $adresse,
                           $code_postal,
                           $ville)
    {
        ob_start();
echo <<<FDD
            <div id="numRecu">
                <p class="important">Reçu numéro {$annee_recu}/{$numero}</p>
            </div>
        </div>

        <div class="cartouche" id="beneficiaire">
            <h3 class="rubrique">Bénéficiaire des versements</h3>
            <p class="important">Association « {$this->nomAsso} »<br />
            {$this->adresseAsso}<br />
            <span class="titre">Objet : </span><span class="libelle">{$this->objetAsso}</span></p>
        </div>

        <div class="cartouche" id="donateur">
            <h3 class="rubrique">Donateur</h3>
            <p>{$nom}<br />
            {$adresse}<br />
            {$code_postal} {$ville}</p>
        </div>

        <div class="cartouche" id="versements">
            <p>Le bénéficiaire reconnaît avoir reçu au titre des dons et versements ouvrant droit à réduction d'impôt :</p>
            <ul>

FDD;

        foreach ($lesMontants as $taux => $montant)
        {
            $this->imprimer_montant($montant,
                                    Utils::getLigneReduction($taux));
        }
        echo "</ul>\n";
        $this->imprimer_description("Date des versements :",
                                    "année {$annee_recu}");
        $this->imprimer_description("Nature du don : ",
                                    "Numéraire");
        $this->imprimer_description("Mode de versement : ",
                                    "chèque et/ou virement");

        // articles du CGI
        $nbArticles = count($this->articlesCGI);
        if ($nbArticles == 1)
        {
            echo "<p>Le bénéficiaire certifie sur l’honneur que les dons et versements qu’il reçoit ouvrent droit à la réduction d'impôt prévue à l’article ";
            printf("%s du code général des impôts</p>\n", $this->articlesCGI[0]);
        }
        else if ($nbArticles > 1)
        {
            echo "<p>Le bénéficiaire certifie sur l’honneur que les dons et versements qu’il reçoit ouvrent droit à la réduction d'impôt prévue aux articles ";
            for ($i = 0; $i < $nbArticles; ++$i) {
                printf("%s", $this->articlesCGI[$i]);
                if ($i < $nbArticles - 2) {
                    echo ", ";
                }
                else if ($i == $nbArticles - 2) {
                    echo " et ";
                }
            }
            echo " du code général des impôts</p>\n";
        }
        echo "</div>\n";

        // cartouche final
        $date = date("j/m/Y");
echo <<<FDD
        <div class="cartouche" id="final">
        <p>{$this->villeAsso} le {$date}<br />
          <img id="signature" src="$this->signature" />
        </p>
        <div>
          <span id="nom">$this->nomResponsable</span><br />
          <span id="fonction">$this->fonctionResponsable</span>
        </div>
        </div>
        </body>
        </html>
FDD;
        $this->html .= ob_get_clean();
    }

    // imprimer un libellé précédé de son titre en gras
    function imprimer_description($titre, $libelle)
    {
        echo <<<FDD
<p><span class="titre">{$titre}</span><span class="libelle"> {$libelle}</span></p>

FDD;
    }

    // imprimer le montant du versement et un libellé
    function imprimer_montant($montant, $libelle = "")
    {
        $valeur = number_format($montant, 2, ',', '');
        echo "<li>la somme de <b>***{$valeur}*** euros</b>";
        if ($libelle != "") {
        echo " ({$libelle})";
        }
        echo "</li>\n";
    }

    protected function entete()
    {
        ob_start();
echo <<<FDD
<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <style>
            @page
            {
                size: A4 portrait;
                margin: 1cm;
            }
            body
            {
                font-family: Serif;
                font-size: 11pt;
                background-color: white;
                width : 19cm;
            }
            #entete
            {
                position : relative;
            }
            #logo
            {
                position : fixed;
                max-width : 4cm;
                max-height : 4cm;
            }
            #titre
            {
                position : fixed;
                margin : 0 4.5cm 0 4.5cm;
                top : 1cm;
                text-align : center;
                font-size : 14pt;
                font-weight: bold;
            }
            #articles
            {
                position : fixed;
                margin : 0 4.5cm 0 4.5cm;       /* idem titre */
                top : 6em;
                text-align : center;
            }
            #numRecu
            {
                position : relative;
                text-align : right;
                top : 8em;
                right: 1em;
            }
            #beneficiaire, #donateur, #versements, #final
            {
                position : relative;
                top : 9em;
            }
            #versements
            {
                position : relative;
                top : 9em;
                border-top: 1px solid rgb(0, 0, 128);
                border-bottom: 1px solid rgb(0, 0, 128);
            }
            .rubrique
            {
                background-color : rgb(200, 200, 250);
                padding : 2mm;
            }
            .titre, .important
            {
                font-weight:bold;
            }
            .libelle
            {
                font-weight: normal;
            }
            #signature
            {
                display: block;
                max-width : 7cm;
                max-height : 4cm;
                margin: 0 auto;
                padding-bottom : 2mm;
            }
        </style>
    </head>
    <body>
        <div class="cartouche" id="entete">
            <img id="logo" src="$this->logoAsso" />
            <p id="titre">Reçu au titre des dons à certains organismes d'intérêt général</p>
            <p id="articles">Articles 200, 238 bis et 978 du code général des impôts</p>

FDD;
        return ob_get_clean();
    }
}
